@@ -16,7 +16,7 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: frame.c,v 1.56 2004/01/23 10:22:26 stux Exp $
+ * $Id: frame.c,v 1.60 2004/04/13 13:47:33 danchr Exp $
  */
 
 /*
@@ -48,11 +48,15 @@
 #include "backpred.h"
 #include "version.h"
 
-static char *libfaacName = FAAC_VERSION " (" __DATE__ ")";
+#if FAAC_RELEASE
+static char *libfaacName = FAAC_VERSION;
+#else
+static char *libfaacName = FAAC_VERSION " (" __DATE__ ") UNSTABLE";
+#endif
 static char *libCopyright =
   "FAAC - Freeware Advanced Audio Coder (http://www.audiocoding.com/)\n"
-  " Portions Copyright (C) 1999,2000,2001 Menno Bakker\n"
-  " Portions Copyright (C) 2002,2003 Krzysztof Nikiel\n"
+  " Copyright (C) 1999,2000,2001  Menno Bakker\n"
+  " Copyright (C) 2002,2003  Krzysztof Nikiel\n"
   "This software is based on the ISO MPEG-4 reference source code.\n";
 
 static const psymodellist_t psymodellist[] = {
@@ -61,9 +65,13 @@ static const psymodellist_t psymodellist[] = {
 };
 
 static SR_INFO srInfo[12+1];
-static const int bwmax = 16000;
-static const double bwfac = 0.45;
 
+// base bandwidth for q=100
+static const int bwbase = 16000;
+// bandwidth multiplier (for quantiser)
+static const int bwmult = 120;
+// max bandwidth/samplerate ratio
+static const double bwfac = 0.45;
 
 
 int FAACAPI faacEncGetVersion( char **faac_id_string,
@@ -224,17 +232,15 @@ int FAACAPI faacEncSetConfiguration(faacEncHandle hEncoder,
 				(double)config->bandWidth * hEncoder->sampleRate / 44100;
 				config->bitRate = (double)config->bitRate * hEncoder->sampleRate / 44100;
 
-		if (config->bandWidth > bwmax)
-		  config->bandWidth = bwmax;
+		if (config->bandWidth > bwbase)
+		  config->bandWidth = bwbase;
 	}
 
     hEncoder->config.bitRate = config->bitRate;
 
     if (!config->bandWidth)
     {
-		config->bandWidth = bwfac * hEncoder->sampleRate;
-		if (config->bandWidth > bwmax)
-			config->bandWidth = bwmax;
+                config->bandWidth = (config->quantqual - 100) * bwmult + bwbase;
     }
 
     hEncoder->config.bandWidth = config->bandWidth;
@@ -308,8 +314,8 @@ faacEncHandle FAACAPI faacEncOpen(unsigned long sampleRate,
     hEncoder->config.useTns = 0;
     hEncoder->config.bitRate = 0; /* default bitrate / channel */
     hEncoder->config.bandWidth = bwfac * hEncoder->sampleRate;
-    if (hEncoder->config.bandWidth > bwmax)
-		hEncoder->config.bandWidth = bwmax;
+    if (hEncoder->config.bandWidth > bwbase)
+		hEncoder->config.bandWidth = bwbase;
     hEncoder->config.quantqual = 100;
     hEncoder->config.psymodellist = (psymodellist_t *)psymodellist;
     hEncoder->config.psymodelidx = 0;
@@ -355,7 +361,7 @@ faacEncHandle FAACAPI faacEncOpen(unsigned long sampleRate,
     }
 
     /* Initialize coder functions */
-	fft_init( &hEncoder->fft_tables );
+	fft_initialize( &hEncoder->fft_tables );
     
 	hEncoder->psymodel->PsyInit(&hEncoder->gpsyInfo, hEncoder->psyInfo, hEncoder->numChannels,
         hEncoder->sampleRate, hEncoder->srInfo->cb_width_long,
@@ -942,6 +948,19 @@ static SR_INFO srInfo[12+1] =
 
 /*
 $Log: frame.c,v $
+Revision 1.60  2004/04/13 13:47:33  danchr
+clarify release <> unstable status
+
+Revision 1.59  2004/04/02 14:56:17  danchr
+fix name clash w/ libavcodec: fft_init -> fft_initialize
+bump version number to 1.24 beta
+
+Revision 1.58  2004/03/17 13:34:20  danchr
+Automatic, untuned setting of lowpass for VBR.
+
+Revision 1.57  2004/03/15 20:16:42  knik
+fixed copyright notice
+
 Revision 1.56  2004/01/23 10:22:26  stux
 *** empty log message ***
 
