@@ -1,6 +1,8 @@
 #ifndef _VERSION_H_
 #define _VERSION_H_
 
-#define FAAC_VERSION "1.23.5"
+#define FAAC_RELEASE 1
+
+#define FAAC_VERSION "1.24"
 
 #endif
