@@ -1,6 +1,6 @@
 /*
 FAAC - codec plugin for Cooledit
-Copyright (C) 2002 Antonio Foranna
+Copyright (C) 2002-2004 Antonio Foranna
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,174 +16,25 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 			
 The author can be contacted at:
-kreel@tiscali.it
+ntnfrn_email-temp@yahoo.it
 */
 
 #include <windows.h>
 #include <shellapi.h>	// ShellExecute
-#include <stdio.h>		// FILE *
-#include <stdlib.h>		// malloc, free
+//#include <stdio.h>		// FILE *
+//#include <stdlib.h>		// malloc, free
 #include "resource.h"
 #include "filters.h"	// CoolEdit
-#include <mp4.h>		// int32_t, ...
-#include <faac.h>
-#include <faad.h>		// FAAD2 version
-#include <win32_ver.h>	// mpeg4ip version
-#include "CRegistry.h"
-#include "Defines.h"	// my defines
-#include "Structs.h"	// my structs
+#include "Cfaac.h"
 
 // *********************************************************************************************
 
 extern HINSTANCE hInst;
 
-// -----------------------------------------------------------------------------------------------
-
-extern BOOL DialogMsgProcDecoder(HWND hWndDlg, UINT Message, WPARAM wParam, LPARAM lParam);
-extern void ReadCfgDec(MY_DEC_CFG *cfg);
-extern void WriteCfgDec(MY_DEC_CFG *cfg);
-
-// *********************************************************************************************
-
-#ifdef	ADTS
-#undef	ADTS
-#define ADTS 1
-#endif
-
-// *********************************************************************************************
-
-typedef struct output_tag  // any special vars associated with output file
-{
-FILE			*aacFile;
-long			Samprate;
-WORD			BitsPerSample;
-WORD			Channels;
-//DWORD			src_size;
-//char			*dst_name;		// name of compressed file
-
-faacEncHandle	hEncoder;
-int32_t			*buffer;
-unsigned char	*bitbuf;
-DWORD			maxBytesOutput;
-long			samplesInput;
-} MYOUTPUT;
-
-
-
 // *********************************************************************************************
 
 
 
-#define SWAP32(x) (((x & 0xff) << 24) | ((x & 0xff00) << 8) \
-	| ((x & 0xff0000) >> 8) | ((x & 0xff000000) >> 24))
-#define SWAP16(x) (((x & 0xff) << 8) | ((x & 0xff00) >> 8))
-
-inline void To32bit(int32_t *buf, BYTE *bufi, int size, BYTE samplebytes, BYTE bigendian)
-{
-int i;
-
-	switch(samplebytes)
-	{
-	case 1:
-		// this is endian clean
-		for (i = 0; i < size; i++)
-			buf[i] = (bufi[i] - 128) * 65536;
-		break;
-		
-	case 2:
-#ifdef WORDS_BIGENDIAN
-		if (!bigendian)
-#else
-			if (bigendian)
-#endif
-			{
-				// swap bytes
-				for (i = 0; i < size; i++)
-				{
-					int16_t s = ((int16_t *)bufi)[i];
-					
-					s = SWAP16(s);
-					
-					buf[i] = ((u_int32_t)s) << 8;
-				}
-			}
-			else
-			{
-				// no swap
-				for (i = 0; i < size; i++)
-				{
-					int s = ((int16_t *)bufi)[i];
-					
-					buf[i] = s << 8;
-				}
-			}
-			break;
-			
-	case 3:
-		if (!bigendian)
-		{
-			for (i = 0; i < size; i++)
-			{
-				int s = bufi[3 * i] | (bufi[3 * i + 1] << 8) | (bufi[3 * i + 2] << 16);
-				
-				// fix sign
-				if (s & 0x800000)
-					s |= 0xff000000;
-				
-				buf[i] = s;
-			}
-		}
-		else // big endian input
-		{
-			for (i = 0; i < size; i++)
-			{
-				int s = (bufi[3 * i] << 16) | (bufi[3 * i + 1] << 8) | bufi[3 * i + 2];
-				
-				// fix sign
-				if (s & 0x800000)
-					s |= 0xff000000;
-				
-				buf[i] = s;
-			}
-		}
-		break;
-		
-	case 4:		
-#ifdef WORDS_BIGENDIAN
-		if (!bigendian)
-#else
-			if (bigendian)
-#endif
-			{
-				// swap bytes
-				for (i = 0; i < size; i++)
-				{
-					int s = bufi[i];
-					
-					buf[i] = SWAP32(s);
-				}
-			}
-			else
-				memcpy(buf,bufi,size*sizeof(u_int32_t));
-		/*
-		int exponent, mantissa;
-		float *bufo=(float *)buf;
-			
-			for (i = 0; i < size; i++)
-			{
-				exponent=bufi[(i<<2)+3]<<1;
-				if(bufi[i*4+2] & 0x80)
-					exponent|=0x01;
-				exponent-=126;
-				mantissa=(DWORD)bufi[(i<<2)+2]<<16;
-				mantissa|=(DWORD)bufi[(i<<2)+1]<<8;
-				mantissa|=bufi[(i<<2)];
-				bufo[i]=(float)ldexp(mantissa,exponent);
-			}*/
-			break;
-	}
-}
-// *********************************************************************************************
 /*
 DWORD PackCfg(MY_ENC_CFG *cfg)
 {
@@ -224,52 +75,6 @@ void UnpackCfg(MY_ENC_CFG *cfg, DWORD dwOptions)
 }*/
 // -----------------------------------------------------------------------------------------------
 
-void ReadCfgEnc(MY_ENC_CFG *cfg) 
-{ 
-CRegistry reg;
-
-	if(reg.openCreateReg(HKEY_LOCAL_MACHINE,REGISTRY_PROGRAM_NAME "\\FAAC"))
-	{
-		cfg->AutoCfg=reg.getSetRegBool("Auto",true);
-		cfg->EncCfg.mpegVersion=reg.getSetRegDword("MPEG version",MPEG4); 
-		cfg->EncCfg.aacObjectType=reg.getSetRegDword("Profile",LOW); 
-		cfg->EncCfg.allowMidside=reg.getSetRegDword("MidSide",true); 
-		cfg->EncCfg.useTns=reg.getSetRegDword("TNS",true); 
-		cfg->EncCfg.useLfe=reg.getSetRegDword("LFE",false);
-		cfg->UseQuality=reg.getSetRegBool("Use quality",false);
-		cfg->EncCfg.quantqual=reg.getSetRegDword("Quality",100); 
-		cfg->EncCfg.bitRate=reg.getSetRegDword("BitRate",0); 
-		cfg->EncCfg.bandWidth=reg.getSetRegDword("BandWidth",0); 
-		cfg->EncCfg.outputFormat=reg.getSetRegDword("Header",ADTS); 
-	}
-	else
-		MessageBox(0,"Can't open registry!",0,MB_OK|MB_ICONSTOP);
-}
-// -----------------------------------------------------------------------------------------------
-
-void WriteCfgEnc(MY_ENC_CFG *cfg) 
-{ 
-CRegistry reg;
-
-	if(reg.openCreateReg(HKEY_LOCAL_MACHINE,REGISTRY_PROGRAM_NAME "\\FAAC"))
-	{
-		reg.setRegBool("Auto",cfg->AutoCfg); 
-		reg.setRegDword("MPEG version",cfg->EncCfg.mpegVersion); 
-		reg.setRegDword("Profile",cfg->EncCfg.aacObjectType); 
-		reg.setRegDword("MidSide",cfg->EncCfg.allowMidside); 
-		reg.setRegDword("TNS",cfg->EncCfg.useTns); 
-		reg.setRegDword("LFE",cfg->EncCfg.useLfe); 
-		reg.setRegBool("Use quality",cfg->UseQuality); 
-		reg.setRegDword("Quality",cfg->EncCfg.quantqual); 
-		reg.setRegDword("BitRate",cfg->EncCfg.bitRate); 
-		reg.setRegDword("BandWidth",cfg->EncCfg.bandWidth); 
-		reg.setRegDword("Header",cfg->EncCfg.outputFormat); 
-	}
-	else
-		MessageBox(0,"Can't open registry!",0,MB_OK|MB_ICONSTOP);
-}
-// -----------------------------------------------------------------------------------------------
-
 #define INIT_CB(hWnd,nID,list,IdSelected) \
 { \
 	for(int i=0; list[i]; i++) \
@@ -301,8 +106,12 @@ CRegistry reg;
 	EnableWindow(GetDlgItem(hWndDlg, IDC_RADIO_ADTS), Enabled); \
 	EnableWindow(GetDlgItem(hWndDlg, IDC_CHK_ALLOWMIDSIDE), Enabled); \
 	EnableWindow(GetDlgItem(hWndDlg, IDC_CHK_USETNS), Enabled); \
+	EnableWindow(GetDlgItem(hWndDlg, IDC_CHK_USELFE), Enabled); \
+	EnableWindow(GetDlgItem(hWndDlg, IDC_CB_QUALITY), Enabled); \
 	EnableWindow(GetDlgItem(hWndDlg, IDC_CB_BITRATE), Enabled); \
 	EnableWindow(GetDlgItem(hWndDlg, IDC_CB_BANDWIDTH), Enabled); \
+	EnableWindow(GetDlgItem(hWndDlg, IDC_RADIO_QUALITY), Enabled); \
+	EnableWindow(GetDlgItem(hWndDlg, IDC_RADIO_BITRATE), Enabled); \
     EnableWindow(GetDlgItem(hWndDlg, IDC_RADIO_MAIN), Enabled); \
     EnableWindow(GetDlgItem(hWndDlg, IDC_RADIO_LOW), Enabled); \
     EnableWindow(GetDlgItem(hWndDlg, IDC_RADIO_LTP), Enabled); \
@@ -337,8 +146,7 @@ BOOL DialogMsgProcAbout(HWND hWndDlg, UINT Message, WPARAM wParam, LPARAM lParam
 					"This program is free software and can be distributed/modifyed under the terms of the GNU General Public License.\n"
 					"This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.\n\n"
 					"Compiled on %s\n",
-				(myFormat->version == FAAC_CFG_VERSION)
-				? myFormat->name : " bad version",
+				(myFormat->version == FAAC_CFG_VERSION) ? myFormat->name : " bad version",
 					__DATE__
 					);
 			SetDlgItemText(hWndDlg, IDC_L_ABOUT, buf);
@@ -349,7 +157,11 @@ BOOL DialogMsgProcAbout(HWND hWndDlg, UINT Message, WPARAM wParam, LPARAM lParam
 		switch(LOWORD(wParam))
 		{
 		case IDOK:
-			EndDialog(hWndDlg, 0);
+			EndDialog(hWndDlg, TRUE);
+			break;
+        case IDCANCEL:
+			// Ignore data values entered into the controls and dismiss the dialog window returning FALSE
+			EndDialog(hWndDlg, FALSE);
 			break;
 		case IDC_AUDIOCODING:
 			ShellExecute(hWndDlg, NULL, "http://www.audiocoding.com", NULL, NULL, SW_SHOW);
@@ -358,7 +170,7 @@ BOOL DialogMsgProcAbout(HWND hWndDlg, UINT Message, WPARAM wParam, LPARAM lParam
 			ShellExecute(hWndDlg, NULL, "http://www.mpeg4ip.net", NULL, NULL, SW_SHOW);
 			break;
 		case IDC_EMAIL:
-			ShellExecute(hWndDlg, NULL, "mailto:kreel@tiscali.it", NULL, NULL, SW_SHOW);
+			ShellExecute(hWndDlg, NULL, "mailto:ntnfrn_email-temp@yahoo.it", NULL, NULL, SW_SHOW);
 			break;
 		}
 		break;
@@ -383,7 +195,7 @@ BOOL DIALOGMsgProc(HWND hWndDlg, UINT Message, WPARAM wParam, LPARAM lParam)
 		char *BandWidth[]={"Auto","Full","4000","8000","11025","16000","22050","24000","32000","44100","48000",0};
 		MY_ENC_CFG cfg;
 
-			ReadCfgEnc(&cfg);
+			Cfaac::getFaacCfg(&cfg);
 
 			INIT_CB(hWndDlg,IDC_CB_QUALITY,Quality,0);
 			INIT_CB(hWndDlg,IDC_CB_BITRATE,BitRate,0);
@@ -468,8 +280,9 @@ BOOL DIALOGMsgProc(HWND hWndDlg, UINT Message, WPARAM wParam, LPARAM lParam)
 				break;
 			}
 
-			CheckDlgButton(hWndDlg,IDC_CHK_AUTOCFG, cfg.AutoCfg);
+			CheckDlgButton(hWndDlg, IDC_CHK_WRITEMP4, cfg.SaveMP4);
 
+			CheckDlgButton(hWndDlg,IDC_CHK_AUTOCFG, cfg.AutoCfg);
 			DISABLE_CTRL(!cfg.AutoCfg);
 		}
 		break; // End of WM_INITDIALOG                                 
@@ -541,7 +354,9 @@ BOOL DIALOGMsgProc(HWND hWndDlg, UINT Message, WPARAM wParam, LPARAM lParam)
 				}
 				cfg.EncCfg.outputFormat=IsDlgButtonChecked(hWndDlg,IDC_RADIO_RAW) ? RAW : ADTS;
 
-				WriteCfgEnc(&cfg);
+				cfg.SaveMP4=IsDlgButtonChecked(hWndDlg, IDC_CHK_WRITEMP4) ? TRUE : FALSE;
+
+				Cfaac::setFaacCfg(&cfg);
 
 				EndDialog(hWndDlg, 1);
 			}
@@ -581,18 +396,22 @@ BOOL DIALOGMsgProc(HWND hWndDlg, UINT Message, WPARAM wParam, LPARAM lParam)
 #define ERROR_FGO(msg) \
 { \
 	if(msg) \
-		MessageBox(hWnd, msg, APP_NAME " plugin", MB_OK|MB_ICONSTOP); \
+	{ \
+	char buf[100]; \
+		sprintf(buf,"FilterGetOptions: %s", msg); \
+		MessageBox(0, buf, APP_NAME " plugin", MB_OK|MB_ICONSTOP); \
+	} \
 	return 0; \
 }
 // -----------------------------------------------------------------------------------------------
 
 DWORD FAR PASCAL FilterGetOptions(HWND hWnd, HINSTANCE hInst, long lSamprate, WORD wChannels, WORD wBitsPerSample, DWORD dwOptions)
 {
-CRegistry	reg;
 long		retVal;
+/*CRegistry	reg;
 BOOL		OpenDialog=FALSE;
 
-/*	if(!reg.openCreateReg(HKEY_LOCAL_MACHINE,REGISTRY_PROGRAM_NAME  "\\FAAD"))
+	if(!reg.openCreateReg(HKEY_LOCAL_MACHINE,REGISTRY_PROGRAM_NAME  "\\FAAD"))
 		ERROR_FGO("Can't open registry!")
 	else
 		if(OpenDialog=reg.getSetRegBool("OpenDialog",FALSE))
@@ -638,7 +457,7 @@ BOOL		OpenDialog=FALSE;
 void FAR PASCAL GetSuggestedSampleType(LONG *lplSamprate, WORD *lpwBitsPerSample, WORD *wChannels)
 {
 	*lplSamprate=0; // don't care
-	*lpwBitsPerSample= *lpwBitsPerSample<=16 ? 0: 16;
+	*lpwBitsPerSample= *lpwBitsPerSample<=16 ? 0 : 16;
 	*wChannels= *wChannels<49 ? 0 : 48;
 }
 // *********************************************************************************************
@@ -648,177 +467,37 @@ void FAR PASCAL CloseFilterOutput(HANDLE hOutput)
 	if(!hOutput)
 		return;
 
-MYOUTPUT *mo;
-
-	GLOBALLOCK(mo,hOutput,MYOUTPUT,return);
-	
-	if(mo->aacFile)
-	{
-		fclose(mo->aacFile);
-		mo->aacFile=0;
-	}
-	
-	if(mo->hEncoder)
-		faacEncClose(mo->hEncoder);
-	
-	FREE(mo->bitbuf)
-	FREE(mo->buffer)
-	
-//	FREE(mi->dst_name);
-	
-	GlobalUnlock(hOutput);
-	GlobalFree(hOutput);
+Cfaac tmp(hOutput); // this line frees memory
 }              
 // *********************************************************************************************
 
-#define ERROR_OFO(msg) \
-{ \
-	if(msg) \
-		MessageBox(0, msg, APP_NAME " plugin", MB_OK|MB_ICONSTOP); \
-	if(hOutput) \
-	{ \
-		GlobalUnlock(hOutput); \
-		CloseFilterOutput(hOutput); \
-	} \
-	return 0; \
-}
-// -----------------------------------------------------------------------------------------------
-
 HANDLE FAR PASCAL OpenFilterOutput(LPSTR lpstrFilename,long lSamprate,WORD wBitsPerSample,WORD wChannels,long lSize, long far *lpChunkSize, DWORD dwOptions)
 {
-HANDLE			hOutput;
-MYOUTPUT		*mo;
-MY_ENC_CFG		cfg;
-DWORD			samplesInput,
-				maxBytesOutput;
+HANDLE	hOutput;
+Cfaac	tmp;
 
-//	if(wBitsPerSample!=8 && wBitsPerSample!=16) // 32 bit audio from cooledit is in unsupported format
-//		return 0;
-	if(wChannels>=49)	// FAAC supports max 48 tracks!
-		return 0;
-
-    if(!(hOutput=GlobalAlloc(GMEM_MOVEABLE|GMEM_SHARE|GMEM_ZEROINIT,sizeof(MYOUTPUT))))
-		ERROR_OFO("Memory allocation error: hOutput");
-	if(!(mo=(MYOUTPUT *)GlobalLock(hOutput)))
-		ERROR_OFO("GlobalLock(hOutput)");
-
-	// open the aac output file 
-	if(!(mo->aacFile=fopen(lpstrFilename, "wb")))
-		ERROR_OFO("Can't create file");
-
-	// use bufferized stream
-	setvbuf(mo->aacFile,NULL,_IOFBF,32767);
-
-	// open the encoder library
-	if(!(mo->hEncoder=faacEncOpen(lSamprate, wChannels, &samplesInput, &maxBytesOutput)))
-		ERROR_OFO("Can't open library");
-
-	if(!(mo->bitbuf=(unsigned char *)malloc(maxBytesOutput*sizeof(unsigned char))))
-		ERROR_OFO("Memory allocation error: output buffer");
-
-	if(!(mo->buffer=(int32_t *)malloc(samplesInput*sizeof(int32_t))))
-		ERROR_OFO("Memory allocation error: input buffer");
-
-	ReadCfgEnc(&cfg);
-	if(!cfg.AutoCfg)
+	if(hOutput=tmp.Init(lpstrFilename,lSamprate,wBitsPerSample,wChannels,lSize))
 	{
-    faacEncConfigurationPtr myFormat=&cfg.EncCfg;
-    faacEncConfigurationPtr CurFormat=faacEncGetCurrentConfiguration(mo->hEncoder);
+	MYOUTPUT *mo;
+		GLOBALLOCK(mo,hOutput,MYOUTPUT,return NULL);
+		*lpChunkSize=mo->samplesInput*(wBitsPerSample>>3); // size of samplesInput
 
-		if(cfg.UseQuality)
-		{
-			myFormat->quantqual=cfg.EncCfg.quantqual;
-			myFormat->bitRate=CurFormat->bitRate;
-		}
-		else
-			if(!myFormat->bitRate)
-				myFormat->bitRate=CurFormat->bitRate;
-			else
-				myFormat->bitRate*=1000;
-
-		switch(myFormat->bandWidth)
-		{
-		case 0:
-			myFormat->bandWidth=CurFormat->bandWidth;
-			break;
-		case 0xffffffff:
-			myFormat->bandWidth=lSamprate/2;
-			break;
-		default: break;
-		}
-
-		if(!faacEncSetConfiguration(mo->hEncoder, myFormat))
-			ERROR_OFO("Unsupported parameters");
+		GlobalUnlock(hOutput);
+		tmp.hOutput=NULL;
 	}
 
-	*lpChunkSize=samplesInput*(wBitsPerSample>>3);
-
-//	mo->src_size=lSize;
-	mo->Samprate=lSamprate;
-	mo->BitsPerSample=wBitsPerSample;
-	mo->Channels=wChannels;
-	mo->samplesInput=samplesInput;
-	mo->maxBytesOutput=maxBytesOutput;
-//	mi->dst_name=strdup(lpstrFilename);
-
-	// init flushing process
-int bytesEncoded, tmp;
-
-    bytesEncoded=faacEncEncode(mo->hEncoder, 0, 0, mo->bitbuf, maxBytesOutput); // initializes the flushing process
-    if(bytesEncoded>0)
-	{
-		tmp=fwrite(mo->bitbuf, 1, bytesEncoded, mo->aacFile);
-		if(tmp!=bytesEncoded)
-			ERROR_OFO("fwrite()");
-	}
-
-	GlobalUnlock(hOutput);
-
-    return hOutput;
+	return hOutput;
 }
 // *********************************************************************************************
 
-#define ERROR_WFO(msg) \
-{ \
-	if(msg) \
-		MessageBox(0, msg, APP_NAME " plugin", MB_OK|MB_ICONSTOP); \
-	if(hOutput) \
-		GlobalUnlock(hOutput); \
-	return 0; \
-}
-// -----------------------------------------------------------------------------------------------
-
 DWORD FAR PASCAL WriteFilterOutput(HANDLE hOutput, unsigned char far *bufIn, long lBytes)
 {
 	if(!hOutput)
 		return 0;
 
-int bytesWritten;
-int bytesEncoded;
-MYOUTPUT far *mo;
+Cfaac tmp;
+DWORD bytesWritten;
 
-	GLOBALLOCK(mo,hOutput,MYOUTPUT,return 0);
-
-int32_t *buf=mo->buffer;
-
-	To32bit(buf,bufIn,mo->samplesInput,mo->BitsPerSample>>3,false);
-
-	// call the actual encoding routine
-	bytesEncoded=faacEncEncode(mo->hEncoder, (int32_t *)buf, mo->samplesInput, mo->bitbuf, mo->maxBytesOutput);
-	if(bytesEncoded>0)
-	{
-		// write bitstream to aac file 
-		bytesWritten=fwrite(mo->bitbuf, 1, bytesEncoded, mo->aacFile);
-		if(bytesWritten!=bytesEncoded)
-			ERROR_WFO("bytesWritten and bytesEncoded are different");
-	}
-	else
-	{
-		if(bytesEncoded<0)
-			ERROR_WFO("faacEncEncode()");
-		bytesWritten=lBytes ? 1 : 0; // bytesWritten==0 stops CoolEdit
-	}
-	
-	GlobalUnlock(hOutput);
-	return bytesWritten;
+	bytesWritten=tmp.processData(hOutput,bufIn,lBytes);
+	return bytesWritten ? bytesWritten : 0x7fffffff; // bytesWritten<=0 stops CoolEdit
 }
